/*
 *
 * $Id: udp_daemon.h,v 1.2.4.26 2000/02/21 06:35:38 ajp Exp $
 *
 * Andrew Pitman
 *
 * pvmsync, a distributed synchronization server:  udp daemon decls.
 *
 * Server that accepts connections (requests) for shared
 * data or synchronization mechanisms continuously and
 * changes some internal state according to the request.
 *
 * Copyright (C) 1999, 2000 Andrew J. Pitman
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */


#ifndef __UDP_DAEMON_H
#define __UDP_DAEMON_H


#ifndef __CONFIG_H
  /* SYMBOL_LENGTH */
#include <config.h>
#endif

/* int16_t */
#include <db.h>


/* All application-level headers start with this as the first byte */
#define UDP_MAGIC_NO              (char) 0x61

/* Protocol version */
#define PROTOCOL_VERS             (char) 0x1

/* Request/Reply field */
#define UDP_REQUEST               (char) 0x1
#define UDP_REPLY                 (char) 0x2

/* Format of data area of datagram.  Includes application-level
   header and data payload.  Entire data portion no less than 16 bytes
   long in multiples of four. */
struct udp_app_dgram {
  char magic,               /* Magic # */
       version,             /* Protocol version */
       obj_type,            /* Object Type */
       length,              /* Total length of 'data' portion */
       dgram_type,          /* Whether request or reply */
       seq_num;             /* 1 byte to distinguish current transaction */
  int16_t num_arg;          /* Optional numeric argument */
  char dp[SYMBOL_LENGTH];   /* String containing object name, null
                               terminated if less than SYMBOL_LENGTH
                               bytes long */
} __attribute__ ((packed));

/* We will maintain an array of these */
struct udp_respond {
  int sock;                       /* Socket to respond on */
  struct sockaddr_in *inet_addr;  /* Address for responses,
                                     filled in by recvfrom(2) */
  struct udp_app_dgram packet;    /* Packet struct for the
                                     incoming message, enough
                                     to hold 64 bit header and
                                     a full symbol name */
};


/* Print usage information and exit*/
void usage __P ((char *program_name)) __attribute__ ((noreturn));

/* Function to process the query received by the udp daemon
   (look up with tcp server), then respond with a datagram
   if appropriate. */
void * proc_query __P ((void *index));

/* free stuff, mark our fd as invalid, and exit (pthread_exit) */
inline void cleanup __P ((void *index));


#endif /* __UDP_DAEMON_H */

